//
//  ADVImportExportPlugin.h
//
//  Base class for plug-ins that handle import and export of files and databases.
//
//  Created by Michael Dupuis
//  Copyright (c) 2006-2008 Advenio, LLC. All rights reserved.
//

/*
 IMPORTANT:  This Advenio software is supplied to you by Advenio, LLC. ("Advenio") in
 consideration of your agreement to the following terms, and your use, installation, 
 modification or redistribution of this Advenio software constitutes acceptance of these 
 terms.  If you do not agree with these terms, please do not use, install, modify or 
 redistribute this Advenio software.
 
 In consideration of your agreement to abide by the following terms, and subject to these 
 terms, Advenio grants you a personal, non-exclusive license, under Advenio's copyrights in 
 this original Advenio software (the "Advenio Software"), to use, reproduce, modify and 
 redistribute the Advenio Software, with or without modifications, in source and/or binary 
 forms; provided that if you redistribute the Advenio Software in its entirety and without 
 modifications, you must retain this notice and the following text and disclaimers in all 
 such redistributions of the Advenio Software.  Neither the name, trademarks, service marks 
 or logos of Advenio, LLC. may be used to endorse or promote products derived from 
 the Advenio Software without specific prior written permission from Advenio. Except as expressly
 stated in this notice, no other rights or licenses, express or implied, are granted by Advenio
 herein, including but not limited to any patent rights that may be infringed by your 
 derivative works or by other works in which the Advenio Software may be incorporated.

 Additionally, the Advenio Software shall not be used to enhance, or be incorporated into,
 any other application that processes, stores or manipulates recipes. By using this Advenio
 Softare you agree to only use it to enhance MacGourmet, or it's related components.
 
 The Advenio Software is provided by Advenio on an "AS IS" basis.  ADVENIO MAKES NO WARRANTIES, 
 EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, 
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE ADVENIO SOFTWARE OR ITS 
 USE AND OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL ADVENIO BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL 
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, 
 REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE ADVENIO SOFTWARE, HOWEVER CAUSED AND 
 WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR 
 OTHERWISE, EVEN IF ADVENIO HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import <Foundation/Foundation.h>
#import "ADVPlugin.h"

#define QUANTITY_KEY		@"QUANTITY"
#define MEASUREMENT_KEY		@"MEASUREMENT"
#define DIRECTION_KEY		@"DIRECTION"
#define DESCRIPTION_KEY		@"DESCRIPTION"
#define IS_DIVIDER_KEY		@"IS_DIVIDER"

@interface ADVImportExportPlugin : ADVPlugin
{
	@private
	
	NSString *_filePath;
	NSString *_recipeDelimiter;
	BOOL _shouldCleanText;
}

/**
 * Should return a dictionary that contains the following elements:
 * 
 * INGREDIENTS  - an array of Ingredient objects
 * PREP_TIMES   - an array of preparatonTime objects
 * TITLE		- a string that containts the recipe title
 * DIRECTIONS   - a string that contains the recipe directions
 * NUTRITION	- a string that contains the nutritional info
 * IMAGE		- an NSImage for the recipe
 * SERVINGS		- an NSNumber that contains the number the recipe serves
 * SOURCE		- a string that contains the source of the recipe
 */
- (NSDictionary *)recipeElements;

/**
 * Returns a dictionary, parsed from the string, containing the elements of an
 * ingredient.
 */
+ (NSDictionary *)ingredientForString:(NSString *)string;

/**
 * Returns a standardized measurement for the measurement passed in. For example, return 'teaspoon' for
 * 'tsp', 't', 'Tsp' or 'ts'.
 */
+ (NSString *)standardizedMeasurement:(NSString *)measurement forQuantity:(NSString *)quantity;

/**
 * Returns a dictionary containing the following keys:
 * QUANTITY
 * MEASUREMENT
 * DIRECTION
 * DESCRIPTION
 */
+ (NSArray *)ingredientsForString:(NSString *)ingredientsStr;

/* Returns YES if the plug-in can export shopping lists */
- (BOOL)supportsShoppingListExport;

/* Returns YES if the plug-in supports imports */
- (BOOL)supportsImport;

/* Returns YES if the plug-in supports exports */
- (BOOL)supportsExport;

/* Returns YES if the plug-in supports exporting a list of items with sublists */
- (BOOL)supportsExportListSublists;

/* Set the full expanded path of the file to be imported or exported. */
- (void)setFilePath:(NSString *)filePath;

/* Returns the full expanded path of the file to be imported or exported. */
- (NSString *)filePath;

/* Returns the total count of recipes to process. Returns -1 if the file could not be processed. */
- (int)importRecipeCount;

/* Returns YES if the plug-in can import the specified file, NO if it cannot. */
- (BOOL)canImportFileWithPath:(NSString *)path;

/* Returns YES if the plug-in can import files using the specified extension, NO if it cannot. */
- (BOOL)canImportFileWithExtension:(NSString *)path;

/* Start the import of the specified file. */
- (BOOL)importFileWithPath:(NSString *)path forController:(id)controller;

/* Start the export of the specified array of MacGourment items */
- (BOOL)exportArray:(NSArray *)arrayOfItems toFile:(NSString *)filePath forController:(id)controller;

/**
 * Count and return the number of occurrences of the specified recipe separator
 * in the string.
 */
- (int)countRecipesInFile:(NSString *)fileString withSeparator:(NSString *)separator;

/**
 * Utility: Converts a file that uses CRLF (DOS) to use just CR
 */
- (NSString *)removeExtraDOSLineEndingCharsFromString:(NSString *)string;

/**
 * Utility: Turns multiple spaces between string components into single spaces.
 */
- (NSString *)trimMultipleSpacesFromString:(NSString *)str;

/**
 * Utility: Turns multiple spaces between string components into single spaces.
 */
- (NSString *)convertMultipleSpacesInString:(NSString *)str intoDelimiter:(NSString *)delim;

/**
 * Utility: Requires a comma delimited keywords string. Parses string and creates
 * categories from a comma delimited list of keywords.
 */
- (NSArray *)categoriesFromKeywords:(NSString *)keywordsStr;

/* Utility: Returns a prep time dictionary for the specified hours and minutes. */
- (NSDictionary *)prepTimesForHours:(int)hours minutes:(int)minutes;

/**
 * Utility: Joins unwrapped text lines and preserves blank lines.
 */
- (void)cleanText:(NSMutableString *)textString;

/**
 * Utility: Trims and joins unwrapped lines and preserves blank lines.
 */
- (NSMutableString *)trimAndJoinLines:(NSMutableString *)textString;

/**
 * Utility: Returns a new string with the first letters of appropriate words capatilized.
 */
- (NSString *)mixedCaseVersionOfString:(NSString *)str;

/**
 * Utility: Modifies the specified mutable string to replace many instances of temperature with
 * a more standard format that includes the unicode symbol for fahrenheit and celcius.
 */
 - (void)standardizeTemperatureFormatInString:(NSMutableString *)str;
 
- (void)setRecipeDelimiter:(NSString *)delimiter;
- (NSString *)recipeDelimiter;

/* Set to YES if the text of directions, notes and nutrition should be cleaned */
- (void)setShouldCleanText:(BOOL)cleanText;
- (BOOL)shouldCleanText;

/* Returns the extension for the export format the plug in supports */
- (NSString *)fileNameExtension;

/**
 * Sets the spotlight comments for a created file. Use to set the comments of a
 * recipe file to the summary, etc.
 */
- (void)setCommentForFile:(NSString *)file toComment:(NSString *)comment;

/* Returns a carbon path for the POSIX path passed in. */
- (NSString *)carbonPathFromPath:(NSString *)path;

/**
 * "Smart" replacement for the deprecated stringWithContentsOfFile. Tries to open
 * the file using different encodings if the default open fails.
 */
- (NSString *)stringWithContentsOfFile:(NSString *)path;

/**
 * "Smart" replacement for the deprecated stringWithContentsOfURL. Tries to open
 * the file using different encodings if the default open fails, returns a parser
 * if it can.
 */
- (NSXMLParser *)xmlParserWithContentsOfURL:(NSURL *)aURL;

@end


/**
 * Protocol for the Import-Export controller selectors to call. These methods
 * are used by the plug-in to interact with the main thread for status, progress, etc.
 */
@protocol ADVImportExportController

/**
 * Plugin selector for setting the next dictionary to add to the database,
 * incrementing the progress indicator, updating the sessions, etc.
 *
 * Returns YES if the import was successful and the process can continue. NO if
 * the import failed or the user canceled the process.
 */
- (BOOL)setNextRecipeDictionary:(NSDictionary *)recipeDictionary;
- (BOOL)setNextWineNoteDictionary:(NSDictionary *)wineNoteDictionary;
- (BOOL)setNextCookingNoteDictionary:(NSDictionary *)cookingNoteDictionary;

/**
 * Notify the main thread that the item was just exported successfully. Used
 * for displaying progress to the user.
 */
- (BOOL)setNextItemExported:(NSString *)itemName;

@end