//
//  ADVPlugin.h
//
//	This is the plug-in class interface. All methods of the plug-in protocol must be
//	implemented. Common methods and instance variables of the plug-in class cluster may
//	also be declared here. Usually the main class itself will never be instantiated.
//	IBOutlets and IBActions are more conveniently declared in the subclasses, unless they're
//	common to all subclasses. In the example, the IBOutlet to the view (and the method that
//	returns it) could have been moved to the main class by using the same outlet name
//	in both associated nib files.
//
//  Created by Michael Dupuis
//  Copyright (c) 2006-2008 Advenio, LLC. All rights reserved.
//

/*
 IMPORTANT:  This Advenio software is supplied to you by Advenio, LLC. ("Advenio") in
 consideration of your agreement to the following terms, and your use, installation, 
 modification or redistribution of this Advenio software constitutes acceptance of these 
 terms.  If you do not agree with these terms, please do not use, install, modify or 
 redistribute this Advenio software.
 
 In consideration of your agreement to abide by the following terms, and subject to these 
 terms, Advenio grants you a personal, non-exclusive license, under Advenio's copyrights in 
 this original Advenio software (the "Advenio Software"), to use, reproduce, modify and 
 redistribute the Advenio Software, with or without modifications, in source and/or binary 
 forms; provided that if you redistribute the Advenio Software in its entirety and without 
 modifications, you must retain this notice and the following text and disclaimers in all 
 such redistributions of the Advenio Software.  Neither the name, trademarks, service marks 
 or logos of Advenio, LLC. may be used to endorse or promote products derived from 
 the Advenio Software without specific prior written permission from Advenio. Except as expressly
 stated in this notice, no other rights or licenses, express or implied, are granted by Advenio
 herein, including but not limited to any patent rights that may be infringed by your 
 derivative works or by other works in which the Advenio Software may be incorporated.

 Additionally, the Advenio Software shall not be used to enhance, or be incorporated into,
 any other application that processes, stores or manipulates recipes. By using this Advenio
 Softare you agree to only use it to enhance MacGourmet, or it's related components.
 
 The Advenio Software is provided by Advenio on an "AS IS" basis.  ADVENIO MAKES NO WARRANTIES, 
 EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, 
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE ADVENIO SOFTWARE OR ITS 
 USE AND OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL ADVENIO BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL 
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, 
 REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE ADVENIO SOFTWARE, HOWEVER CAUSED AND 
 WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR 
 OTHERWISE, EVEN IF ADVENIO HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#import <Foundation/Foundation.h>
#import "PluginInterface.h"

#define ADVPLUGIN_RECIPE		102
#define ADVPLUGIN_WINE_NOTE		103
#define ADVPLUGIN_COOKING_NOTE 	104
#define ADVPLUGIN_SHOPPING_LIST	112
#define ADVPLUGIN_LIST			200

@interface ADVPlugin : NSObject<PAPluginProtocol>
{
	@private
	
	NSString *_name; //	the name to be used in the user interface
	id theObject;	 //	the plug-in manager object
	
	BOOL _pluginIsRegistered;
}

/** The plug-in manager should be passed in or anObject */
+ (ADVPlugin *)pluginFor:(id)anObject;

/** Init plugin with the plug-in manager object and the unique identifier name for the plugin. */
- (id)initWithObject:(id)anObject pluginName:(NSString *)name;

/** Returns the english name of the plug-in */
- (NSString *)pluginName;

/** Returns the integer identifer tag for this plug-in. This has be be unique.  */
- (int)pluginTag;

/** Returns the domain for this plug-in. */
- (NSString *)pluginDomain;

/** Process selection change notifications using the sender and the dictionary. */
- (void)selectionDidChangeNotification:(id)sender selectionInfo:(NSDictionary *)theInfo;

/** Returns the plug-in manager object. */
- (id)pluginManager;

/** Returns YES if the plug-in is registered at startup, NO if it isn't and needs to do it's own registration. */
- (BOOL)pluginIsRegistered;
- (void)setPluginIsRegistered:(BOOL)registered;

/** Returns YES if the plugin is a demo or beta and it has expired. NO if everything is fine. */
+ (BOOL)pluginHasExpired;

/** Returns a string that can be used to instruct the user on how to use the plug-in. */
- (NSString *)helpString;

/**
 * Current version of MacGourmet passed in in form 2.x.x.x
 *  Returns YES if plug-in will work with specified version or higher, NO if it will not.
 */
+ (BOOL)worksWithMacGourmetVersion:(NSString *)version;

/** Override this selector to specify a minimum version other than the default */
+ (NSArray *)minimumVersionComponents;

/** "Tag" selector that identifies any newly built plug-ins as 3.0+ plug-ins. */
+ (BOOL)worksWithMG3;

@end

@protocol MGAppProtocol

////////////////////
// ADVPluginManager
////////////////////

/**
 * Returns the plug-in with the specified tag, so that plug-ins can share data and
 * functionality.
 */
- (NSObject<PAPluginProtocol> *)pluginWithTag:(int)tag;

/**
 * Returns the object with the specified key and object type. To get a dictionary 
 * with the keys and their values, call exportDictionary on the object returned.
 */
- (id)objectWithKey:(int)theKey objectType:(int)theType;

/**
 * Returns YES if the plug-in needs to check for a registration. Returns NO if the plug-in
 * registration is covered by the current MacGourmet serial number (One serial number for everything).
 */
- (BOOL)pluginNeedsToDoRegistration:(int)pluginIDTag;

/**
 * Prints an array of plug-in items that respond to the htmlPrintPage selector
 * with this complete signature:
 * - (NSString *)htmlPrintPage
 */
- (BOOL)printPluginItems:(NSArray *)theItems;

/////////////////////////////////////////////////////////////////////
// Application items. These methods can be called on returned items.
/////////////////////////////////////////////////////////////////////

/**
 * Dictionary that contains the item information.
 *
 *****************************b
 * RECIPE (NSDictionary)
 * TYPE: ADVPLUGIN_RECIPE
 *
 * KEYS:
 *
 * NAME - The recipe title (NSString)
 * SERVINGS - Servings amount (NSNumber)
 * YIELD - Yield info (NSString)
 * SOURCE - Recipe source (NSString)
 * SUMMARY - The recipe summary or introduction (NSString)
 * DIRECTIONS - (NSString)
 * KEYWORDS - String containing keywords list (NSString)
 * NOTE - (NSString)
 * COURSE_ID - The unique id for the course (NSNumber) [Not published]
 * COURSE_NAME - The name of the course (NSString)
 * CUISINE_ID - The unique id of the cuisine (NSNumber) [Not published]
 * DIFFICULTY - The number difficulty value (NSNumber) [0-5]
 * MEASUREMENT_SYSTEM - U.S. or Metric
 * NUTRITION - The nutrition summary (NSString)
 * PREP_TIMES [Not published]
 * TYPE - item type [ADVPLUGIN_RECIPE, ADVPLUGIN_WINE_NOTE, ADVPLUGIN_COOKING_NOTE]
 * URL - URL associated with item
 *
 * INGREDIENTS - NSArray of ingredient dictionaries
 *		INGREDIENT - NSDictionary
 *			DESCRIPTION - (NSString)
 *			DIRECTION - (NSString)
 *			IS_DIVIDER - The ingredient is a divider row [0 or 1]
 *			IS_MAIN - The ingredient is a main ingredient [0 or 1]
 *			MEASUREMENT - (NSString)
 *			QUANTITY - (NSString)       
 *
 * CATEGORIES - NSArray of category dictionaries
 *		CATEGORY_ID - unique database key [Not published]
 *		ITEM_TYPE_ID - the type of item the category is for
 *		NAME - Category name
 *		USER_ADDED - Is the category a user added one [0 or 1]
 *
 *****************************
 * WINE NOTE (NSDictionary)
 * TYPE: ADVPLUGIN_WINE_NOTE
 *
 * KEYS:
 *
 * WINERY - (NSString)
 * VINTAGE - (NSString) 
 * REGION - (NSString)
 * SUBREGION - (NSString)
 * STYLE - (NSString)
 * PRICE - (NSString)
 * TASTING_NOTES - (NSString)
 * WINEMAKER_NOTES - (NSString)
 * WINEMAKER_URL - (NSString)
 * WINE_TYPE - (NSNumber)	[(Red, 2), (White, 5), (Pink, 3), (Kosher, 1), (Dessert, 6), (Bubbly, 4)
 *							// Red
 *							(7, Cabernet Sauvignon), (8, Chianti), (9, Italian Red)
 *							(10, Merlot), (11, Other Red Wine), (12, Pinot Noir)
 *							(13, Red Bordeaux), (14, Sangiovese), (15, Syrah/Shiraz)
 *							(16, Zinfandel), (17, Barbera), (18, Cabernet/Merlot)
 *							(19, Blend), (20, Nebbiolo), (21, Petite Sirah)
 *							(22, Red Burgundy), (23, Tempranillo)
 *							// Whites
 *							(24, Chardonnay), (25, Pinot Gris/Grigio), (26, Chenin Blanc)
 *							(27, Gewurztraminer), (28, Italian White), (29, Muscat)
 *							(30, Other White Wine), (31, Pinot Blanc), (32, Riesling)
 *							(33, Sauvignon/Fume Blanc), (34, Semillon), (35, Viognier)
 *							(36, White Bordeaux), (37, White Burgundy)
 *							// Dessert, Fortified & Fruit
 *							(38, Madeira), (39, Other Dessert), (40, Port), (41, Sherry)
 *							// Bubbly
 *							(42, Champagne), (43, Sparkling)
 *							// Kosher
 *							(44, Kosher Cabernet), (45, Kosher Chardonnay), (46, Kosher Merlot)
 *							(47, Kosher Riesling), (48, Kosher Sauvignon Blanc), (50, Kosher Syrah)
 *							(51, Kosher White Zinfandel), (52, Sparkling)]
 * PURCHASE_DATE - (NSString)
 * PURCHASE_LOCATION - (NSString)
 * DATE_OPENED - (NSString)
 * VARIETAL - (NSString)
 * NUM_ON_HAND - (NSNumber)
 * MATURATION_DATE - (NSString)
 * RATING - (NSNumber) [0 - 5]
 * TYPE - item type [ADVPLUGIN_RECIPE, ADVPLUGIN_WINE_NOTE, ADVPLUGIN_COOKING_NOTE]
 *
 *****************************
 * COOKING NOTE (NSDictionary)
 * TYPE: ADVPLUGIN_COOKING_NOTE
 *
 * KEYS:
 *
 * NAME - The recipe title (NSString)
 * NOTE_TEXT - (NSString)
 * KEYWORDS - (NSString)
 * SOURCE - (NSString)
 * TYPE - item type [ADVPLUGIN_RECIPE, ADVPLUGIN_WINE_NOTE, ADVPLUGIN_COOKING_NOTE]
 */
- (NSMutableDictionary *)exportDictionary;

/**
 * Reset the cached information in an item, to conserve memory.
 */
- (void)resetMissingItems;

/**
 * Returns the text of the string version of the object.
 */
- (NSString *)stringVersionObject;

@end