//
//  ADVToolPlugin.h
//
//  The base class for tool plug-ins.
//
//  Copyright (c) 2007 Advenio. All rights reserved.
//

/*
 IMPORTANT:  This Advenio software is supplied to you by Advenio, LLC. ("Advenio") in
 consideration of your agreement to the following terms, and your use, installation, 
 modification or redistribution of this Advenio software constitutes acceptance of these 
 terms.  If you do not agree with these terms, please do not use, install, modify or 
 redistribute this Advenio software.
 
 In consideration of your agreement to abide by the following terms, and subject to these 
 terms, Advenio grants you a personal, non-exclusive license, under Advenio's copyrights in 
 this original Advenio software (the "Advenio Software"), to use, reproduce, modify and 
 redistribute the Advenio Software, with or without modifications, in source and/or binary 
 forms; provided that if you redistribute the Advenio Software in its entirety and without 
 modifications, you must retain this notice and the following text and disclaimers in all 
 such redistributions of the Advenio Software.  Neither the name, trademarks, service marks 
 or logos of Advenio, LLC. may be used to endorse or promote products derived from 
 the Advenio Software without specific prior written permission from Advenio. Except as expressly
 stated in this notice, no other rights or licenses, express or implied, are granted by Advenio
 herein, including but not limited to any patent rights that may be infringed by your 
 derivative works or by other works in which the Advenio Software may be incorporated.

 Additionally, the Advenio Software shall not be used to enhance, or be incorporated into,
 any other application that processes, stores or manipulates recipes. By using this Advenio
 Softare you agree to only use it to enhance MacGourmet, or it's related components.
 
 The Advenio Software is provided by Advenio on an "AS IS" basis.  ADVENIO MAKES NO WARRANTIES, 
 EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, 
 MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE, REGARDING THE ADVENIO SOFTWARE OR ITS 
 USE AND OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL ADVENIO BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL OR CONSEQUENTIAL 
 DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
 OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, 
 REPRODUCTION, MODIFICATION AND/OR DISTRIBUTION OF THE ADVENIO SOFTWARE, HOWEVER CAUSED AND 
 WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE), STRICT LIABILITY OR 
 OTHERWISE, EVEN IF ADVENIO HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#define ADVPLUGIN_DragPboardType	@"ADVPLUGIN_DragPboardType"

#define NOTIFY_PLUGINS_SELECTION_CHANGED	@"NOTIFY_PLUGINS_SELECTION_CHANGED"
#define NOTIFY_PLUGINS_ITEM_DELETED			@"NOTIFY_PLUGINS_ITEM_DELETED"
#define NOTIFY_PLUGINS_LOOKUP_ITEM			@"NOTIFY_PLUGINS_LOOKUP_ITEM"
#define NOTIFY_PLUGINS_SELECT_ITEM			@"NOTIFY_PLUGINS_SELECT_ITEM"
#define NOTIFY_PLUGINS_ITEM_ADDED			@"NOTIFY_PLUGINS_ITEM_ADDED"
#define NOTIFY_PLUGINS_ITEM_CHANGED			@"NOTIFY_PLUGINS_ITEM_CHANGED"

#import <Cocoa/Cocoa.h>
#import "ADVPlugin.h"

// Tools should register for NSApplicationWillTerminateNotification if any housekeeping
// needs to be done before the application is quit.

// To select an item in the recipe box, fire a notification:
//
// [[NSNotificationCenter defaultCenter]
//		postNotificationName:NOTIFY_PLUGINS_SELECT_ITEM object:self
//		userInfo: [NSDictionary dictionaryWithObjectsAndKeys:
//			[NSNumber numberWithInt: recipePrimaryKey], @"KEY", 
//			[NSNumber numberWithInt:ADVPLUGIN_RECIPE], @"TYPE", nil]];

@interface ADVToolPlugin : ADVPlugin
{
	@public
	id panelController;	// The main window controller object (if there is one)
}

/**
 * Returns the controller object for the main window.
 */
- (id)panelController;

/**
 * Sets the controller object for the main window.
 */
- (void)setPanelController:(id)theController;

/**
 * Returns the submenu NSMenuItem to be added to the Tools menu, or to the Menubar
 */
- (NSMenuItem *)menuWithKeyEquivalent:(NSString *)keyEquivalent;

/**
 * Returns the preferred key equivalent for the plug-in. The plug-in manager
 * can override this request if the key equivalent is already taken by another plug-in.
 */
- (NSString *)preferredKeyEquivalent;

/**
 * Returns the title for the Tools menu menu item, if no submenu is needed.
 */
- (NSString *)titleForMenuItem;

/**
 * Returns an NSImage for the toolbar, if there is one
 */
- (NSImage *)toolbarImage;

/**
 * Returns YES if the submenu item should be added to the main Menubar, NO if it
 * should be added to the Tools menu as a submenu.
 */
- (BOOL)menuShouldBeAddedToMenubar;

/**
 * Subclasses should return an NSView that can be installed into the Preferences
 * window in the application. Default implementation returns nil.
 */
- (NSView *)preferencesPanelView;

/** Return the height necessary to display the preferences pane in the Preferences window */
- (float)preferencesPaneViewHeight;

/**
 * For those plug-ins that need to load data, load it if necessary.
 */
- (void)loadDataIfNecessary;

/**
 * Set the database path, if application wants to control where the tools plug-in
 * stores its data.
 */
- (void)setDatabasePath:(NSString *)databasePath;

/** Returns the user defaults key used to look up the serial number for a plug-in. */
+ (NSString *)defaultsRegistrationKey;

/** Returns the path to the plug-in resource folder with the specified resource type for the preferred localization */
+ (NSString *)localizedPathToResourceFolder:(NSString *)directory forResourceType:(NSString *)resourceType forClass:(Class)objClass;

/** 
  * Returns YES if the plug-in supports lookup, NO if it doesn't. Subclasses should implement.
  */
- (BOOL)supportsLookup;

/** Returns YES if the plug-in supports drag and drop, NO if it doesn't. */
- (BOOL)supportsDragAndDrop;

/**
 * Register for drag and drops from the application.
 *
 * To add support for dragged and dropped items, this registers for the ADVPLUGIN_DragPboardType
 * drag type. Every drag and drop will include an array of dictionaries with the following:
 *
 *	[handler registerForDraggedTypes:[NSArray arrayWithObjects: ADVPLUGIN_DragPboardType,
 *									<any other drag types you're interested in>, nil]];
 *
 *  Do this in acceptDrop for the view
 *  NSPasteboard *pboard = [info draggingPasteboard];
 *	if ([pboard availableTypeFromArray:[NSArray arrayWithObjects:ADVPLUGIN_DragPboardType, nil]] != nil)
 *  {
 *		Do your processing here
 *  }
 *
 *  ADVPLUGIN_DragPboardType defines a type for a drag array with one or more
 *  dictionaries with these keys:
 *  KEY: The primary key of the object being dragged
 *  TYPE: The type of the object being dragged
 *  NAME: The name of the object
 */
- (void)registerForDragAndDrop:(id)handlerObject;

/**
 *  If the plug-in wants to get item lookup notifications, it should register with
 *  the default NSNotificationCenter for the NOTIFY_PLUGINS_LOOKUP_ITEM notification.
 *	[[NSNotificationCenter defaultCenter]
 *		addObserver:self selector:@selector(handleItemLookup:)
 *		name:NOTIFY_PLUGINS_LOOKUP_ITEM object:nil];
 *
 *  The userInfo dictionary will contain a dictionary with the following key:
 *  SEARCH: The string to search for
 */
- (void)registerForItemLookupNotification;

- (void)unregisterForItemLookupNotification;

/** Subclasses should implement this to handle the item lookup notification. */
- (void)handleItemLookup:(NSNotification *)notification;

/**
 *  If the plug-in wants to get selection change notifications, it should register with
 *  the default NSNotificationCenter for the NOTIFY_PLUGINS_SELECTION_CHANGED notification.
 *	[[NSNotificationCenter defaultCenter]
 *		addObserver:self selector:@selector(handleSelectionDidChange:)
 *		name:NOTIFY_PLUGINS_SELECTION_CHANGED object:nil];
 *
 *  The userInfo dictionary will contain a dictionary with the following keys:
 *  KEY: The primary key of the object
 *  TYPE: The type of the object
 *  NAME: The name of the object
 */
- (void)registerForSelectionChangeNotification;

- (void)unregisterForSelectionChangeNotification;

/** Subclasses should implement this to handle the selection change notification. */
- (void)handleSelectionWasChanged:(NSNotification *)notification;

/**
 *  If the plug-in wants to get item change notifications, it should register with
 *  the default NSNotificationCenter for the NOTIFY_PLUGINS_ITEM_CHANGED notification.
 *	[[NSNotificationCenter defaultCenter]
 *		addObserver:self selector:@selector(handleItemWasChanged:)
 *		name:NOTIFY_PLUGINS_ITEM_CHANGED object:nil];
 *
 *  The userInfo dictionary will contain a dictionary with the following keys:
 *  KEY: The primary key of the object
 *  TYPE: The type of the object
 *  NAME: The name of the object
 */
- (void)registerForItemChangedNotification;

- (void)unregisterForItemChangedNotification;

/** Subclasses should implement this to handle the item changed notification. */
- (void)handleItemWasChanged:(NSNotification *)notification;

/**
 *  If the plug-in wants to get item deletion notifications, it should register with
 *  the default NSNotificationCenter for the NOTIFY_PLUGINS_ITEM_DELETED notification.
 *	[[NSNotificationCenter defaultCenter]
 *		addObserver:self selector:@selector(handleItemWasDeleted:)
 *		name:NOTIFY_PLUGINS_ITEM_DELETED object:nil];
 *
 *  The userInfo dictionary will contain a dictionary with the following keys:
 *  KEY: The primary key of the object
 *  TYPE: The type of the object
 *  NAME: The name of the object
 */
- (void)registerForItemDeletedNotification;

- (void)unregisterForItemDeletedNotification;

/** Subclasses should implement this to handle the item deletion notification. */
- (void)handleItemWasDeleted:(NSNotification *)notification;

/**
 *  If the plug-in wants to get item added notifications, it should register with
 *  the default NSNotificationCenter for the NOTIFY_PLUGINS_ITEM_ADDED notification.
 *	[[NSNotificationCenter defaultCenter]
 *		addObserver:self selector:@selector(handleItemWasAdded:)
 *		name:NOTIFY_PLUGINS_ITEM_ADDED object:nil];
 *
 *  The userInfo dictionary will contain a dictionary with the following keys:
 *  KEY: The primary key of the object
 *  TYPE: The type of the object
 *  NAME: The name of the object
 */
- (void)registerForItemAddedNotification;

- (void)unregisterForItemAddedNotification;

/** Subclasses should implement this to handle the selection change notification. */
- (void)handleItemWasAdded:(NSNotification *)notification;

@end

//
//  ADVToolPluginPrintItemProtocol
//  The protocol that things plug-ins want to print must adhere to 
//

@protocol ADVToolPluginPrintItemProtocol

/** Returns the html page for printing, by filling in the template in templateStr */
- (NSMutableString *)fillInTemplate:(NSMutableString *)templateStr;

/** Returns the plist key used to load the print templates */
- (NSString *)printKeyForPluginItem;

/** Returns the path to the templates used for printing plug-in items */
- (NSString *)printingTemplatesPath;

/** The unique name to be used for the print file name (in the cache) */
- (NSString *)printFileName;

@end

//
//  ADVToolPreferencesViewProtocol
//

@protocol ADVToolPreferencesViewProtocol

- (void)loadPreferences;

@end